use std::fs::File;
use std::io::{self, prelude::*, BufReader};
use std::collections::HashMap;
use regex::Regex;

fn find_S(line: &str) -> Vec<(i32, i32)> {
    let re = Regex::new(r"\d+").unwrap();
    let matches: Vec<_> = re.find_iter(line).map(|m| m.range()).collect();
    let mut return_vec: Vec<(i32, i32)> = vec![];
    for i in matches {
        return_vec.push((i.start as i32, i.end as i32));
        // println!("first {}, Last {}", i.start, i.end); 
        // println!("number {}", line.get(i.start..i.end).unwrap().parse::<u32>().unwrap()); 
    }
    return return_vec;
}

fn find_next_location(wall_of_text: &Vec<String>, current_location: (i32, i32), previous_location: (i32, i32), left: bool, map: &mut Vec<Vec<i32>>) -> (i32, i32)
{
    let mut sign = 1;
    if left { sign = -1;}
    let current_char = wall_of_text[current_location.0 as usize].get(current_location.1 as usize..current_location.1 as usize+1).unwrap();
    println!("current_char {}", current_char);
    match current_char{
        "|" => {
                map[current_location.0 as usize][current_location.1 as usize] = sign*(previous_location.0 - current_location.0);
                (current_location.0 + (current_location.0 - previous_location.0) , current_location.1)
        },
        "-" => {
                map[current_location.0 as usize][current_location.1 as usize] = 3;
                (current_location.0, current_location.1 + (current_location.1 - previous_location.1))
        },
        "L" => {
            if current_location.0 == previous_location.0 {
                map[current_location.0 as usize][current_location.1 as usize] = sign;
                (current_location.0-1 , current_location.1)
            }
            else
            {
                map[current_location.0 as usize][current_location.1 as usize] = -sign;
                (current_location.0 , current_location.1+1)
            }
        },
        "J" => {
            if current_location.0 == previous_location.0 {
                map[current_location.0 as usize][current_location.1 as usize] = sign;
                (current_location.0-1 , current_location.1)
            }
            else
            {
                map[current_location.0 as usize][current_location.1 as usize] = -sign;
                (current_location.0 , current_location.1-1)
            }
        }
        "7" => {
            if current_location.0 == previous_location.0 {
                map[current_location.0 as usize][current_location.1 as usize] = -sign;
                (current_location.0+1 , current_location.1)
            }
            else
            {
                map[current_location.0 as usize][current_location.1 as usize] = sign;
                (current_location.0 , current_location.1-1)
            }
        }
        "F" => {
            if current_location.0 == previous_location.0 {
                map[current_location.0 as usize][current_location.1 as usize] = -sign;
                (current_location.0+1 , current_location.1)
            }
            else
            {
                map[current_location.0 as usize][current_location.1 as usize] = sign;
                (current_location.0 , current_location.1+1)
            }
        }
        "." => (current_location.0 , current_location.1),
        &_ => (current_location.0 , current_location.1),
    }

}

fn set_map_val(wall_of_text: &Vec<String>, map: &mut Vec<Vec<i32>>, loc: (i32, i32)){
    let current_char = wall_of_text[loc.0 as usize].get(loc.1 as usize..loc.1 as usize+1).unwrap();
    map[loc.0 as usize][loc.1 as usize]= match current_char {
        "-" => 3,
        &_ => 1,
    };
}

fn process_text(wall_of_text: Vec<String>) -> u32 {
    let mut starting_location = (0,0);
    let mut map: Vec<Vec<_>> = vec![];
    for (line_count, line) in wall_of_text.clone().into_iter().enumerate() {
        let x = line.find("S");
        map.push(vec![0; line.len()]);
        if x.is_some(){
            println!("S is located at {} {}", line_count, x.unwrap());
            starting_location = (line_count as i32, x.unwrap() as i32);
            // break;
        }
    }
    let mut cardinal_directions: Vec<_> = vec![];
    let mut heads: Vec<(i32, i32)> = vec![];
    let mut prev_locs: Vec<(i32, i32)> = vec![];
    prev_locs.push(starting_location);
    prev_locs.push(starting_location);
    cardinal_directions.push(wall_of_text[(starting_location.0 - 1)  as usize].get(starting_location.1 as usize..starting_location.1 as usize+1).unwrap());
    cardinal_directions.push(wall_of_text[(starting_location.0 + 1) as usize].get(starting_location.1 as usize..starting_location.1 as usize+1).unwrap());
    cardinal_directions.push(wall_of_text[starting_location.0 as usize].get((starting_location.1 + 1) as usize..(starting_location.1 + 1) as usize+1).unwrap());
    cardinal_directions.push(wall_of_text[starting_location.0 as usize].get((starting_location.1 - 1) as usize..(starting_location.1 - 1) as usize+1).unwrap());

    if match cardinal_directions[0]{
        "|" => true,
        "-" => false,
        "L" => false,
        "J" => false,
        "7" => true,
        "F" => true,
        "." => false,
        &_ => false,
    } {
        map[starting_location.0 as usize][starting_location.1 as usize] = -1;
        heads.push(((starting_location.0 - 1), (starting_location.1)))};

    if match cardinal_directions[1]{
        "|" => true,
        "-" => false,
        "L" => true,
        "J" => true,
        "7" => false,
        "F" => false,
        "." => false,
        &_ => false,
    } {
        if heads.len() == 0 {
            map[starting_location.0 as usize][starting_location.1 as usize] = 1;
        }
        heads.push(((starting_location.0 + 1), (starting_location.1)))};

    if match cardinal_directions[2]{
        "|" => false,
        "-" => true,
        "L" => false,
        "J" => true,
        "7" => true,
        "F" => false,
        "." => false,
        &_ => false,
    } {
        if heads.len() == 0 {
            map[starting_location.0 as usize][starting_location.1 as usize] = 3;
        }
        heads.push(((starting_location.0), (starting_location.1+1)))};

    if match cardinal_directions[3]{
        "|" => false,
        "-" => true,
        "L" => true,
        "J" => false,
        "7" => false,
        "F" => true,
        "." => false,
        &_ => false,
    } {heads.push(((starting_location.0), (starting_location.1-1)))};

    let mut count = 1;
    // set_map_val(&wall_of_text, &mut map, starting_location);
    // set_map_val(&wall_of_text, &mut map, heads[0]);
    // set_map_val(&wall_of_text, &mut map, heads[1]);
    // map[starting_location.0 as usize][starting_location.1 as usize]=1;
    // map[heads[0].0 as usize][heads[0].1 as usize]=1;
    // map[heads[1].0 as usize][heads[1].1 as usize]=1;

    while heads[0] != heads[1]
    {
        let temp0 = heads[0];
        let temp1 = heads[1];
        heads[0] = find_next_location(&wall_of_text, heads[0], prev_locs[0], true, &mut map);
        heads[1] = find_next_location(&wall_of_text, heads[1], prev_locs[1], false, &mut map);
        // set_map_val(&wall_of_text, &mut map, heads[0]);
        // set_map_val(&wall_of_text, &mut map, heads[1]);

        // map[heads[0].0 as usize][heads[0].1 as usize]=1;
        // map[heads[1].0 as usize][heads[1].1 as usize]=1;
        prev_locs[0] = temp0;
        prev_locs[1] = temp1;
        count += 1;
    }
    heads[0] = find_next_location(&wall_of_text, heads[0], prev_locs[0], true, &mut map);
    heads[1] = find_next_location(&wall_of_text, heads[1], prev_locs[1], false, &mut map);

    //now to count tiles inside
    count = 0;
    let mut inside;
    for (counti, i) in map.clone().into_iter().enumerate() {
        inside = false;
        for (countj, j) in i.into_iter().enumerate() {
            let ch = wall_of_text[counti as usize].get(countj as usize..countj as usize+1).unwrap();
            if (j == 1 || j == -1) {

                if ch == "F"{
                    inside = !inside;
                    // inside = true;
                }
                // if ch == "J"{
                //     inside = !inside;
                //     // inside = true;
                // }
                if ch == "7" {
                    inside = !inside;
                    // inside = false;
                }
                // if ch == "L" {
                //     inside = !inside;
                //     // inside = false;
                // }
                if ch == "|" {
                    inside = !inside;
                }
            }
            // if ch == "L" || ch == "J" {
            //     inside = false;
            // }
            // // if j == 1 {
            //     inside = false;
            // }
            if j== 0 {
                if inside {
                    map[counti][countj] = 2;
                    count += 1;
                }
            }
        }
        println!("{:?}", map[counti]);
    }
    
    
    return count;
}

fn main() -> io::Result<()>  {
    // let file = File::open("../test.txt")?;
    let file = File::open("../input.txt")?;
    let reader = BufReader::new(file);
    let answer: u32 = process_text(
        reader.lines().collect::<io::Result<Vec<String>>>().expect("failed")
    );
    println!("{}", answer);

    Ok(())
}
