use std::fs::File;
use std::io::{self, prelude::*, BufReader};
use std::collections::HashMap;
use regex::Regex;

fn conv_i32_to_usize(var :i32) -> usize {
    usize::try_from(var).unwrap()
}

fn process_ray(wall_of_text: &Vec<String>, mut rays_vec: Vec<[[i32; 2]; 2]>, ray_index: usize, mut past_rays: Vec<[[i32; 2]; 2]>) -> (Vec<[[i32; 2]; 2]>, Vec<[[i32; 2]; 2]>)
{
    let current_char = wall_of_text[conv_i32_to_usize(rays_vec[ray_index][0][1])].get(conv_i32_to_usize(rays_vec[ray_index][0][0])..conv_i32_to_usize(rays_vec[ray_index][0][0])+1).unwrap();
    // println!("current_char {}", current_char);
    let mut split = false;
    past_rays.push(rays_vec[ray_index]);
    let mut new_ray: [[i32; 2]; 2] = [[0,0],[0,0]];
    match current_char{
        "|" => { 
            // If not moving left right, we must be moving up, down
            if rays_vec[ray_index][1][0] == 0 {
                rays_vec[ray_index][0][0] = rays_vec[ray_index][0][0] + rays_vec[ray_index][1][0];
                rays_vec[ray_index][0][1] = rays_vec[ray_index][0][1] + rays_vec[ray_index][1][1];
            }
            else{
                if rays_vec[ray_index][0][1] - 1 >= 0 {
                    split = true;
                    new_ray = [[rays_vec[ray_index][0][0], rays_vec[ray_index][0][1] - 1], [0, -1]];
                }
                rays_vec[ray_index] = [[rays_vec[ray_index][0][0], rays_vec[ray_index][0][1] + 1], [0, 1]];
            }
        },
        "-" => {
            // If not movine left right, we must be moving up, down
            if rays_vec[ray_index][1][0] == 0 {
                if rays_vec[ray_index][0][0] - 1 >= 0 {
                    split = true;
                    new_ray = [[rays_vec[ray_index][0][0] - 1  , rays_vec[ray_index][0][1]], [-1, 0]];
                }
                rays_vec[ray_index] = [[rays_vec[ray_index][0][0] + 1  , rays_vec[ray_index][0][1]], [1, 0]];
            }
            else{
                rays_vec[ray_index][0][0] = rays_vec[ray_index][0][0] + rays_vec[ray_index][1][0];
                rays_vec[ray_index][0][1] = rays_vec[ray_index][0][1] + rays_vec[ray_index][1][1];
            }
        },
        "\\" => {
            let ray = rays_vec[ray_index];
            rays_vec[ray_index][1][0] = ray[1][1];
            rays_vec[ray_index][1][1] = ray[1][0];
            rays_vec[ray_index][0][0] = rays_vec[ray_index][0][0] + rays_vec[ray_index][1][0];
            rays_vec[ray_index][0][1] = rays_vec[ray_index][0][1] + rays_vec[ray_index][1][1];
        },
        "/" => {
            let ray = rays_vec[ray_index];
            rays_vec[ray_index][1][0] = -ray[1][1];
            rays_vec[ray_index][1][1] = -ray[1][0];
            rays_vec[ray_index][0][0] = rays_vec[ray_index][0][0] + rays_vec[ray_index][1][0];
            rays_vec[ray_index][0][1] = rays_vec[ray_index][0][1] + rays_vec[ray_index][1][1];
        },
        "." => {
            rays_vec[ray_index][0][0] = rays_vec[ray_index][0][0] + rays_vec[ray_index][1][0];
            rays_vec[ray_index][0][1] = rays_vec[ray_index][0][1] + rays_vec[ray_index][1][1];
        },
        &_ => panic!(),
    }
    
    // println!("ray_index {}", ray_index);
    let ray = rays_vec[ray_index];
    let mut remove_ray = false;
    if ((ray[0][0] < 0) || (ray[0][0] == wall_of_text.len().try_into().unwrap()) ||
        (ray[0][1] < 0) || ray[0][1] == wall_of_text[0].len().try_into().unwrap())
    {
        // println!("REMOVING {}", ray_index);
        remove_ray = true;
    }
    else{
        for past_ray in &past_rays {
            if past_ray == &ray {
                // println!("REMOVING duplicate ray {}", ray_index);
                remove_ray = true;
                break;
            }
        }
    }
    
    if remove_ray{
        rays_vec.remove(ray_index);
    }

    if split {
        rays_vec.push(new_ray);
    }

    

    (rays_vec, past_rays)

}

fn process_text(wall_of_text: Vec<String>) -> u32 { 
    let mut light_rays: Vec<[[i32; 2]; 2]> = vec![];
    let mut past_rays: Vec<[[i32; 2]; 2]> = vec![];
    light_rays.push([[0,0], [1, 0]]);
    while (light_rays.len() > 0) {
        // println!("{light_rays:?}");
        // let mut temp_light_rays = .clone();
        for ray in (0..light_rays.len()).rev() {
            (light_rays, past_rays) = process_ray(&wall_of_text, light_rays, ray, past_rays);
        }

        // light_rays = temp_light_rays;
    }

    let mut count = 0;
    let mut energized_set: Vec<[i32; 2]> = vec![];
    // energized_set.push([0,0]);
    for i in &past_rays{
        if i[0][0] < 0 || i[0][0] >= wall_of_text.len().try_into().unwrap() ||
        i[0][1] < 0 || i[0][1] >= wall_of_text[0].len().try_into().unwrap()
    {
        panic!();
    }
        if !energized_set.contains(&i[0]){
            energized_set.push(i[0]);
            count += 1;
        }
    }
    
    // println!("Len of past rays {}", count);
    // println!("energized_set {energized_set:?}");
    println!("energized_set {:?}", energized_set.len());
    println!("past_rays {:?}", past_rays.len());
    
    return 0;
}

fn main() -> io::Result<()>  {
    // let file = File::open("../test.txt")?;
    let file = File::open("../input.txt")?;
    let reader = BufReader::new(file);
    let answer: u32 = process_text(
        reader.lines().collect::<io::Result<Vec<String>>>().expect("failed")
    );
    println!("{}", answer);

    Ok(())
}
