use std::fs::File;
use std::io::{self, prelude::*, BufReader};
use std::collections::HashMap;
use regex::Regex;

struct Node{
    straights: u8,
    heat: u32,
    location: (isize, isize),
    direction: (isize, isize),
    path: Vec<(isize, isize)>,
}


fn check_bound(x: isize , y: isize, max_x: isize, max_y: isize) -> bool {
    return !(x >= max_x || x < 0 || y >= max_y || y < 0);
}

fn check_finished(nodes: &Vec<Node>, max_x: isize, max_y: isize) -> bool {
    let mut min_node = std::u32::MAX;
    let mut min_idx = 0;
    for count in 0..nodes.len() {
        if nodes[count].heat < min_node {
            min_node = nodes[count].heat;
            min_idx = count;
        }
    }
    println!("heat {}", nodes[min_idx].heat);
    println!("num_nodes {}", nodes.len());
    println!("locs {} {}", nodes[min_idx].location.0, nodes[min_idx].location.1);
    println!("path {:?}", nodes[min_idx].path);
    return nodes[min_idx].location.0 == max_x-1 && nodes[min_idx].location.1 == max_y-1;

}

fn loc_not_in_locs(loc: Node, locs: &Vec<Node>) -> bool{
    for current_loc in locs{
        if (loc.location.0 == current_loc.location.0 && loc.location.1 == current_loc.location.1)// && loc.heat >= current_loc.heat+10)// && loc.direction.0 == current_loc.direction.0 && loc.direction.1 == current_loc.direction.1 && loc.straights == current_loc.straights && loc.heat >= current_loc.heat)
        {
            return false;
        }
    }
    return true;
}

fn take_step(map: &Vec<Vec<u32>>,  nodes: &mut Vec<Node>, visited_loc: &mut Vec<Node>) {
    let mut min_node = std::u32::MAX;
    let nodes_len = nodes.len();
    let mut min_idx = 0;
    for count in 0..nodes.len() {
        if nodes[count].heat < min_node {
            min_node = nodes[count].heat;
            min_idx = count;
        }
    }


    // Add a straight to the list
    if nodes[min_idx].straights < 2{
        let new_loc = (nodes[min_idx].location.0 + nodes[min_idx].direction.0, nodes[min_idx].location.1 + nodes[min_idx].direction.1);
        if check_bound(new_loc.0, new_loc.1, map[0].len().try_into().unwrap(), map.len().try_into().unwrap())
        {
            let new_dir = nodes[min_idx].direction;
            let new_heat = nodes[min_idx].heat + map[(new_loc.0) as usize][(new_loc.1) as usize];
            let straight = nodes[min_idx].straights + 1;
            let mut new_path = nodes[min_idx].path.clone();
            new_path.push(nodes[min_idx].location);
            let new_node: Node = {Node { straights: straight, heat: new_heat, location: new_loc, direction: new_dir, path: new_path}};
            if loc_not_in_locs({Node { straights: straight, heat: new_heat, location: new_loc, direction: new_dir, path: vec![]}}, &visited_loc){
                // println!("Adding {:?}", new_node.location);
                // visited_loc.push({Node { straights: straight, heat: new_heat, location: new_loc, direction: new_dir, path: vec![]}});
                nodes.push(new_node);
            }
        }
    }

    // Add a turn left and right
    for i in [-1, 1]{
        // println!("{}", i);
        let new_dir = (i * nodes[min_idx].direction.1, i * nodes[min_idx].direction.0);
        let new_loc = (nodes[min_idx].location.0 + new_dir.0, nodes[min_idx].location.1 + new_dir.1);
        if check_bound(new_loc.0, new_loc.1, map[0].len().try_into().unwrap(), map.len().try_into().unwrap())
        {
            let new_heat = nodes[min_idx].heat + map[(new_loc.0) as usize][(new_loc.1) as usize];
            let straight = 0;
            let mut new_path = nodes[min_idx].path.clone();
            new_path.push(nodes[min_idx].location);
            let new_node: Node = {Node { straights: straight, heat: new_heat, location: new_loc, direction: new_dir, path: new_path}};
            if loc_not_in_locs({Node { straights: straight, heat: new_heat, location: new_loc, direction: new_dir, path: vec![]}}, &visited_loc){
                // println!("Adding {:?}", new_node.location);
                // visited_loc.push({Node { straights: straight, heat: new_heat, location: new_loc, direction: new_dir, path: vec![]}});
                nodes.push(new_node);
            }
        }
    }
    let n_loc = nodes[min_idx].location;
    let n_s = nodes[min_idx].straights;
    let n_dir = nodes[min_idx].direction;
    let n_heat = nodes[min_idx].heat;
    visited_loc.push({Node { straights: n_s, heat: n_heat, location: n_loc, direction: n_dir, path: vec![]}});
    // if nodes[min_idx].path.len() > 3{
        // let purge_location = nodes[min_idx].path[nodes[min_idx].path.len()-3-1];
        // nodes.remove(min_idx);
        for i in (0..nodes.len()).rev(){
            if nodes[i].location.0 == n_loc.0 && nodes[i].location.1 == n_loc.1 && nodes[i].straights == n_s && nodes[i].direction.0 == n_dir.0 && nodes[i].direction.1 == n_dir.1 //&& nodes[i].heat >= n_heat
            {
                // println!("REMOVING {:?}", nodes[i].location);
                nodes.remove(i);
            }
        }
    // }
    // else{
    // nodes.remove(min_idx);

    // }
    // println!("REMOVING {:?}", nodes[min_idx].location);

    // return nodes;
}

fn process_line(map: &Vec<Vec<u32>> ) -> i64 {
    let mut nodes: Vec<Node> = vec![];
    let mut visited_locs: Vec<Node> = vec![];
    let mut finished = false;
    nodes.push(Node { straights: 0, heat: 0, location: (0, 0), direction: (1, 0), path: vec![]});
    nodes.push(Node { straights: 0, heat: 0, location: (0, 0), direction: (0, 1), path: vec![]});
    while !finished {
        take_step(&map, &mut nodes, &mut visited_locs);
        finished = check_finished(&nodes, map[0].len().try_into().unwrap(), map.len().try_into().unwrap());
    }

    // let numbers = find_numbers(&line);
    return 0;
}

fn main() -> io::Result<()>  {
    let mut sum = 0;
    let mut map: Vec<Vec<u32>> = vec![];
    // let file = File::open("../test.txt")?;
    let file = File::open("../input.txt")?;
    let reader = BufReader::new(file);
    for line in reader.lines() {
        map.push(line?.to_string().chars().map(|m| m.to_digit(10).unwrap()).collect());
    }

    sum = process_line(&map);
    // println!("{map:?}");
    // println!("{}", sum);

    Ok(())
}
