use std::fs::File;
use std::io::{self, prelude::*, BufReader};
use regex::Regex;

fn get_number(line: &str) -> u32 {
    let re = Regex::new(r"\d+").unwrap();
    let num: u32 = re.find(line).unwrap().as_str().parse().unwrap();
    return num;
}

static POSSIBLE_CUBES: [(&str, u32); 3] = [("red", 12), ("blue", 14), ("green", 13)];

fn process_line(line: String) -> u32 {
    let parts = line.split(&[':']).collect::<Vec<&str>>();
    let game_number: u32 = get_number(parts[0]);
    // Parts is a vector of strings? where the first index will be the game
    // and all consecutive indeces will be the game "sets"
    let sets = parts[1].split(&[';']);
    for set in sets {
        let colors = set.split(", ");
        for color in colors {
            let mut found = false; // flag to sanity check, and make sure that all colors are ones that we expect
            for pos in POSSIBLE_CUBES {
                if color.contains(pos.0) {
                    found = true;
                    if pos.1 < get_number(color) {
                        return 0;
                    }
                }
            }
            if found != true {
                println!("{} was present in Game {}", color, game_number);
                return 0;
            }
        }
    }
    return game_number;
}

fn main() -> io::Result<()>  {
    let mut sum: u32 = 0;
    let file = File::open("../input.txt")?;
    let reader = BufReader::new(file);
    for line in reader.lines() {
        sum = process_line(line?) + sum;
    }
    println!("{}", sum);

    Ok(())
}
