use std::fs::File;
use std::io::{self, prelude::*, BufReader};
use regex::Regex;

fn find_numbers(line: &str) -> Vec<i32> {
    let re = Regex::new(r"\d+").unwrap();
    let matches: Vec<_> = re.find_iter(line).map(|m| m.as_str().parse::<i32>().unwrap()).collect();
    let mut return_vec: Vec<i32> = vec![];
    for i in matches {
        return_vec.push(i);
        println!("number {}", i); 
    }
    return return_vec;
}

fn find_gear(line: &str) -> Vec<i32> {
    let re = Regex::new(r"[*]").unwrap();
    let matches: Vec<_> = re.find_iter(line).map(|m| m.range()).collect();
    let mut return_vec: Vec<i32> = vec![];
    for i in matches {
        return_vec.push((i.start as i32).try_into().unwrap());
        println!("first {}, Last {}", i.start, i.end); 
        println!("symbol {}", line.get(i.start..i.end).unwrap()); 
    }
    return return_vec;
}

fn clip(val: i32, min:i32, max: i32) -> usize {
    if val < min 
    {
        return min.try_into().unwrap();
    }
    if val > max 
    {
        return max.try_into().unwrap();
    }
    return val.try_into().unwrap();
}

fn get_adjacent_gears(coordinates: (usize, usize, usize, usize), text: &Vec<String>) -> (bool, i32)
{
    let re = Regex::new(r"[^A-Za-z0-9.]").unwrap();
    let mut window_string = String::new();
    let window = &text[coordinates.0..(coordinates.1+1)];
    for i in window {
        let mut n = coordinates.2;
        let mut temp_string = String::new();
        temp_string.push(i.chars().nth(n).unwrap());
        while n > 0 {
            n -= 1;
            let character = i.chars().nth(n).unwrap();
            temp_string.insert(0, character);
            if !character.is_numeric() {
                break;
            }
        }

        n = coordinates.2;
        while n < coordinates.3-1    {
            n += 1;
            let character = i.chars().nth(n).unwrap();
            temp_string.push(character);
            if !character.is_numeric() {
                break;
            }

        }
        
        window_string.push_str(&temp_string);
    
    }
    let number_vec = find_numbers(&window_string);
    let gear_ratio;
    if number_vec.len() == 2
    {
        gear_ratio = number_vec[0]*number_vec[1];
    }
    else
    {
        gear_ratio = 0;
    }


    println!("{}", window_string);
    return (re.is_match(&window_string), gear_ratio);
}


fn process_text(wall_of_text: Vec<String>) -> i32 {
    let mut sum: i32 = 0;

    for (line_count, line) in wall_of_text.clone().into_iter().enumerate() {
        // let x = find_numbers(&line);
        let y = find_gear(&line);
        for i in y {
            let coor = (
                clip(i32::try_from(line_count).unwrap()-1, 0, i32::try_from(wall_of_text.len()).unwrap()-1),
                clip(i32::try_from(line_count).unwrap()+1, 0, i32::try_from(wall_of_text.len()).unwrap()-1),
                i.try_into().unwrap(), 
                line.len(),
            );

            let (engine_part, ratio) = get_adjacent_gears(coor, &wall_of_text);
            if engine_part 
            {
                sum += ratio;
            }
        }
    }
    // let _throw_way = get_adjacent_gears((2,2,4,4), wall_of_text);
    return sum;
}

fn main() -> io::Result<()>  {
    let file = File::open("../test.txt")?;
    // let file = File::open("../input.txt")?;
    let reader = BufReader::new(file);
    let answer: i32 = process_text(
        reader.lines().collect::<io::Result<Vec<String>>>().expect("failed")
    );
    println!("{}", answer);

    Ok(())
}
