use std::fs::File;
use std::io::{self, prelude::*, BufReader};
use regex::Regex;

fn find_numbers(line: &str) -> Vec<u64> {
    let re = Regex::new(r"\d+").unwrap();
    let matches: Vec<_> = re.find_iter(line).map(|m| m.as_str().parse::<u64>().unwrap()).collect();
    let mut return_vec: Vec<u64> = vec![];
    for i in matches {
        return_vec.push(i);
    }
    return return_vec;
}

fn process_line(line: String) -> u64 {
    let parts = line.split(&[':']).collect::<Vec<&str>>();
    let sets = parts[1].split("|").collect::<Vec<&str>>();
    let winning_numbers = find_numbers(&sets[0]);
    let mut card_sum = 0;
    let my_numbers = find_numbers(&sets[1]);
    for num in &my_numbers
    {
       for winning_num in &winning_numbers 
       {
           if num == winning_num { card_sum +=1; }
       }
    }
    card_sum
}

fn main() -> io::Result<()>  {
    let mut sum: u64 = 0;
    let file = File::open("../input.txt")?;
    let reader = BufReader::new(file);
    let mut instances: Vec<u64> = vec![];
    for (count, line) in reader.lines().enumerate() {
        if count >= instances.len() { instances.push(1);}
        let num = process_line(line?);
        for i in 0..num{
            let idx = count + usize::try_from(i).unwrap() + 1;
            if idx >= instances.len() { instances.push(1+instances[count]);}
            else {instances[idx] += instances[count];}
        }
        sum += instances[count];
    }
    println!("{}", sum);

    Ok(())
}
