use std::fs::File;
use std::io::{self, prelude::*, BufReader};
use regex::Regex;


fn find_numbers(line: &str) -> Vec<u64> {
    let re = Regex::new(r"\d+").unwrap();
    let matches: Vec<_> = re.find_iter(line).map(|m| m.as_str().parse::<u64>().unwrap()).collect();
    let mut return_vec: Vec<u64> = vec![];
    for i in matches {
        return_vec.push(i);
        println!("number {}", i); 
    }
    return return_vec;
}

fn process_seed(seed: u64, translation_table: &Vec<Vec<Vec<u64>>>) -> (u64, u64)
{
    let mut val = seed;
    let mut skip_num = std::u64::MAX;
    for block in translation_table
    {
        for line in block
        {
            if (val >= line[1]) && (val < (line[1] + line[2]))
            {
                let skip_contender = (line[1] + line[2]) - val;
                val = line[0] + (val - line[1]);
                if skip_contender < skip_num {skip_num = skip_contender;}
                break;
            }
        }
    }
    (val, skip_num)
}

fn process_text(wall_of_text: Vec<String>) -> u64 {
    let mut minimum: u64 = std::u64::MAX;
    let mut translation_table: Vec<Vec<Vec<u64>>> = vec![];
    let seeds: Vec<u64> = find_numbers(&wall_of_text[0]);
    let mut line_count = 3; //Start on line 3 to skip  the seeds header
    while line_count < wall_of_text.len() {
        let mut table_block: Vec<Vec<u64>> = vec![];
        while line_count < wall_of_text.len() {
            if wall_of_text[line_count].contains(":")
            {
                break;
            }
            if !wall_of_text[line_count].is_empty() {
                let table_line = find_numbers(&wall_of_text[line_count]);
                table_block.push(table_line);
            }
            else{println!("EMPTY LINE")};
            line_count += 1;
        }
        translation_table.push(table_block);
        line_count += 1;
    }

    for seed_group in 0..seeds.len()/2 {
        let mut seed_number = 0;
        let base = seeds[seed_group*2];
        while seed_number < seeds[seed_group*2+1] {
            let (val, skip_number) = process_seed(base+seed_number, &translation_table);
            seed_number += skip_number;
            if val < minimum { minimum = val; }
        }
    }
    return minimum;
}

fn main() -> io::Result<()>  {
    let file = File::open("../input.txt")?;
    let reader = BufReader::new(file);
    let answer: u64 = process_text(
        reader.lines().collect::<io::Result<Vec<String>>>().expect("failed")
    );
    println!("{}", answer);

    Ok(())
}
